package	jp.co.sds_corp.ratemixer.fx.mixer;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import jp.co.sds_corp.fxutil.CcyPair;
import jp.co.sds_corp.fxutil.ForexCurve;
import jp.co.sds_corp.fxutil.ForexRate;
import jp.co.sds_corp.fxutil.IRateLoader;
import jp.co.sds_corp.fxutil.MasterCache;
import jp.co.sds_corp.ratemixer.fx.FxRate;

public class RateManager
{
	MasterCache master = MasterCache.getInstance();
	HashMap<CcyPair,RateHolder>	mapHolder = new HashMap<CcyPair,RateHolder>();
	HashMap<CcyPair,Vector<IRateChangeListener>>	mapLsn = new HashMap<CcyPair,Vector<IRateChangeListener>>();
	Vector<IRateChangeListener>	vecLsn = new Vector<IRateChangeListener>();
	IRateLoader	loader = new Loader();

	public RateManager() {
		Map<String,CcyPair> map;
		map = master.mapCcyPair();
		Iterator<CcyPair> itr = map.values().iterator();
		while ( itr.hasNext()) {
			CcyPair pair = (CcyPair )itr.next();
			if ( !pair.isFeed()) {
				continue;
			}
			RateHolder holder = new RateHolder( this, pair );
			mapHolder.put( pair, holder );
		}
	}

	public void addRateChangeListener( CcyPair pair, IRateChangeListener lsn ) {
		Vector<IRateChangeListener> vec = mapLsn.get( pair );
		if ( vec == null ) {
			vec = new Vector<IRateChangeListener>();
			mapLsn.put( pair, vec );
		}
		vec.add( lsn );
	}

	public void rateReceived( FxRate rate ) {
		CcyPair pair = rate.getCcyPair();
		Vector<IRateChangeListener> vec = mapLsn.get( pair );
		if ( vec == null ) {
			return;
		}
		for ( int i = 0; i < vec.size(); i++ ) {
			IRateChangeListener lsn = (IRateChangeListener )vec.get( i );
			lsn.rateChanged( rate );
		}
	}

	public void addRateChangeListener( IRateChangeListener lsn ) {
		vecLsn.add( lsn );
	}

	public void fireRateChange( FxRate rate ) {
		for ( int i = 0; i < vecLsn.size(); i++ ) {
			IRateChangeListener lsn = (IRateChangeListener )vecLsn.get( i );
			lsn.rateChanged( rate );
		}
	}

	public IRateLoader getLoader() {
		return loader;
	}

	public class Loader implements IRateLoader
	{
		/**
		* spotレート取得
		*/
		public ForexRate getSpotRate( CcyPair pair ) throws Exception {
			RateHolder holder = (RateHolder )mapHolder.get( pair );
			if ( holder == null ) {
				return null;
			}

			ForexCurve curve = holder.getForexCurve();
			ForexRate ret = curve.getSpotRate();
			return ret;
		}

		/**
		* forwardレート取得
		*/
		public ForexCurve getForexCurve( CcyPair pair ) throws Exception {
			RateHolder holder = (RateHolder )mapHolder.get( pair );
			if ( holder == null ) {
				return null;
			}

			ForexCurve curve = holder.getForexCurve();
			return curve;
		}
	}

}
